#include <stdio.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <utime.h>

// Sparsely synchronise the first file over the second file.
// Copyright (c) Ewan Parker 2007, 2010, 2013.
// $Id: sync.c,v 1.10 2013/09/14 21:01:43 root Exp $
// Main program.

#include "syncutl.h"
#include "syncchk.h"
#include "syncrun.h"

int main(int argc, char *argv[]) {
   int rc = 1;
   struct stat s_stat_buffer, d_stat_buffer;
   struct utimbuf s_utime_buffer, d_utime_buffer;
   char *s_file_name, *d_file_name;
   FILE *sf, *df;

   if (argc != 3)
      fprintf(stderr, "Usage: %s source_file destination_file\n", argv[0]);
   else {
      s_file_name = argv[1]; d_file_name = argv[2];

      if (stat_file(s_file_name, &s_stat_buffer) && stat_file(d_file_name, &d_stat_buffer)) {
         if (sync_possible(&s_stat_buffer, &d_stat_buffer)) {
            if (sync_required(&s_stat_buffer, &d_stat_buffer)) {
               sf = fopen(s_file_name, "r");
               if (errno)
                  fprintf(stderr, "Open of %s failed with %d.\n", s_file_name, errno);
               else {
                  df = fopen(d_file_name, "r+");
                  if (errno)
                     fprintf(stderr, "Open of %s failed with %d.\n", d_file_name, errno);
                  else {
                     rc = sync_data(sf, df);
                     fclose (df);
                  }
                  fclose (sf);
                  if (rc == 0) rc = sync_metadata(s_file_name, d_file_name);
               }
            }
            else rc = 0; // Sync not required.
         }
         else rc = 2; // Sync not possible.
      }
   }
   return rc;
}
