#include <stdio.h>
#include <sys/stat.h>
#include <unistd.h>

// Sparsely synchronise the first file over the second file.
// Copyright (c) Ewan Parker 2007, 2010, 2013.
// $Id: syncchk.c,v 1.5 2013/09/15 08:51:39 root Exp $
// Synchronisation checks.

#include "syncchk.h"

int sync_possible(struct stat *ss, struct stat *ds) {
   int rc = 1;
   if (!S_ISREG(ss->st_mode)) {
      printf("Source is not a regular file.\n");
      rc = 0;
   }
   if (!S_ISREG(ds->st_mode)) {
      printf("Destination is not a regular file.\n");
      rc = 0;
   }
   if (ss->st_size != ds->st_size) {
      printf("File sizes %d and %d differ.\n", ss->st_size, ds->st_size);
      rc = 0;
   }
   return rc;
}

int sync_required(struct stat *ss, struct stat *ds) {
   int rc = 0;
   if (ss->st_mode != ds->st_mode) {
      printf("Access modes %#o and %#o differ.\n", ss->st_mode, ds->st_mode);
      rc = 1;
   }
   if (ss->st_uid != ds->st_uid) {
      printf("UIDs %d and %d differ.\n", ss->st_uid, ds->st_uid);
      rc = 1;
   }
   if (ss->st_gid != ds->st_gid) {
      printf("GIDs %d and %d differ.\n", ss->st_gid, ds->st_gid);
      rc = 1;
   }
   if (ss->st_mtime != ds->st_mtime) {
      printf("Modified times %d and %d differ.\n", ss->st_mtime, ds->st_mtime);
      rc = 1;
   }
   return rc;
}
