#include <stdio.h>
#include <errno.h>
#include <string.h>

#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <utime.h>

// Sparsely synchronise the first file over the second file.
// Copyright (c) Ewan Parker 2007, 2010, 2013.
// $Id: syncrun.c,v 1.7 2013/09/15 08:52:10 root Exp $
// Run-time code.

#include "syncrun.h"

int get_block_size(FILE *f);

int sync_data(FILE *sf, FILE *df) {
   int block_size = get_block_size(df);
   char sb[block_size], db[block_size];
   unsigned long int r = 0, w = 0, rc = 0, wc = 0;
   int b;

   printf("Blocksize %d.\nSynchronising", block_size); fflush(stdout);
   while (!feof(sf)) {
      b = fread(&sb, 1, block_size, sf);
      if (b > 0) {
         r += b; rc += b;
         if (rc >= 1024*1024*1024) {
            if (wc > 0) printf("!"); else printf("."); fflush(stdout);
            rc = 0; wc = 0;
         }
         fread(&db, 1, block_size, df);
         if (memcmp(&sb, &db, b)) {
            fseek(df, -b, SEEK_CUR);
            fwrite(&sb, 1, b, df);
            w += b; wc += b;
         }
      }
   }
   printf(".\nRead %ld bytes.\n", r);
   printf("Wrote %ld bytes.\n", w);
   return 0;
}

int sync_metadata(char *sfname, char *dfname) {
   int rc = 0;
   struct stat stat_buffer;
   struct utimbuf utime_buffer;

   if (stat(sfname, &stat_buffer)) {
      fprintf(stderr, "Cannot read metadata from %s.\n", sfname);
      rc = 1;
   }
   else {
      if (chown(dfname, stat_buffer.st_uid, stat_buffer.st_gid)) {
         fprintf(stderr, "Cannot update UID/GID metadata on %s.\n", dfname);
         rc = 1;
      }
      if (chmod(dfname, stat_buffer.st_mode)) {
         fprintf(stderr, "Cannot update permission metadata on %s.\n", dfname);
         rc = 1;
      }
      utime_buffer.actime = stat_buffer.st_atime;
      utime_buffer.modtime = stat_buffer.st_mtime;
      if (utime(dfname, &utime_buffer)) {
         fprintf(stderr, "Cannot update time stamp metadata on %s.\n", dfname);
         rc = 1;
      }
   }
   return rc;
}

int get_block_size(FILE *f) {
   int b = DEFBLOCKSIZE;
   struct stat stat_buffer;
   if (fstat(fileno(f), &stat_buffer))
      fprintf(stderr, "Cannot determine block size: using default.\n");
   else
      b = stat_buffer.st_blksize;
   return b;
}
